<?

/**
 * Representation of collections as a model. 
 * 
 * @author Sourcemap Team
 * @package sourcemap
 * @subpackage models
 */

class CollectionsModel extends Model {
	
	function CollectionsModel() {
		parent::Model();
		$this->load->model("ObjectsModel", "om");
	}
	
	/**
	* Adds a new collection to the database.
	* @param string $name The name of the collection
	* @param string $description The description of the collection
	* @param string $criterion The collection criterion
	* @return int The unique ID of the collection
	*/
	function createCollection($name, $description, $criterion) {
	  $this->load->helper('url');
	  $data = array ('name' => addslashes($name),
			 'description'=> addslashes($description),
			 'criterion' => addslashes($criterion),
			 'slug' => url_title(strtolower($name)));
	  $this->db->insert("collections", $data);
	  return $this->db->insert_id();
	}
	
	/**
	* Returns collections whose names are like the search term.
	* @param string $collectionName The name of the collection to search
	* @return array The collection(s) matching the search term
	*/
	function searchCollectionsByName($collectionName) {
		$this->db->from('collections');
		$this->db->like('name', $collectionName);
		$this->db->orderBy('name');
		$query = $this->db->get();
		return $query->result();
	}
	
	/**
	* Gets $limit collections, in order by $sort, starting at $offset
	* @param int $limit The maximum number of collections to return.
	* @param int $offset The index at which to start returning collections
	* @param string $sort The sorting method to use
	* @return array The sorted collections in the given range.
	*/
	function getCollectionsInRange($limit=null, $offset=null, $sort = 'name ASC') {
		$this->db->select('*');
		$this->db->from('collections');
		$this->db->order_by($sort);
		$this->db->limit($limit, $offset);
		$query = $this->db->get();
		$collections = $query->result();
		
		foreach($collections as $collection) {
			$collection->objectCount = $this->getObjectCount($collection->id);
		}
		
		return $collections;
	}
	
	/**
	* Returns a collection whose unique id matches $id
	* @param int $id The unique id
	* @return object The collection matching the given id.
	*/
	function getCollectionByID($id) {
		$this->db->select('*');
		$this->db->from('collections');
		$this->db->where('id', $id);
		$query = $this->db->get();
		return array_pop($query->result());
	}
	
	/**
	* Returns a collection whose slug matches $slug
	* @param int $slug The collection slug to match
	* @return object The collection matching the given slug.
	*/	
	function getCollectionBySlug($slug) {
		$this->db->select('*');
		$this->db->from('collections');
		$this->db->where('slug', $slug);
		$query = $this->db->get();
		return array_pop($query->result());
	}
	
	/**
	* Returns a collection whose name matches $name.
	* Note: Collection names are not guaranteed programmatically to be unique
	* @param string $name The name of the collection to match
	* @return object The collection whose name matches $name
	*/
	function getCollectionIDByName($name) {
		$this->db->select('id');
		$this->db->from('collections');
		$this->db->where('name', $name);
		$query = $this->db->get();
		return array_pop($query->result());
	}
	
	/**
	 * Returns the number of objects in a collection
	 * @param int $collectionID The ID of the collection to get an object count for
	 * @return int The number of objects in the collection
	 */
	function getObjectCount($collectionID) {
		$this->db->select('*');
		$this->db->from('collections');
		$this->db->where('id', $collectionID);
		$query = $this->db->get();
		
		//Get the number of objects that match this collection's criterion
		//(i.e. the number of objects in this collection)
		$collection = array_pop($query->result());
		return count($this->getObjectsByStatement($collection->criterion));
	}
	
	/**
	 * Returns $n random collections from all of the collections in the table
	 * @param int $n The number of random collections to return
	 * @return array The set of $n random collections
	 */
	function getNRandomCollections($n) {
		$this->db->select('*');
		$this->db->from('collections');
		$query = $this->db->get();
		$allCollections = $query->result();
		
		//Make sure we're not asking for more collections than we have
		if($n > count($allCollections)) {
			$n = count($allCollections);
		}
		
		$randomCollections = array();
		shuffle($allCollections);
		for($i = 0; $i < $n; $i++) {
			$randomCollections[] = $allCollections[$i];
		}
		
		return $randomCollections;
	}
	
	/**
	 * Parses a collection criterion into something that is more easily assembled into an SQL query
	 * @param string $statement The collection criterion
	 * @return array An associative array representing a collection criterion
	 */
	function parseStatement($statement) {
		//Statements can be of the following forms:
		//Ownership:*; Match:$field $relation $value [$field $relation $value...]
		//Ownership:$type $id; Match:*
		//Ownership:$type $id; Match:$field $relation $value [$field $relation $value...]
		//where Ownership defines a $type with ID #$id's ownership of an object
	    //and Match defines a $field of object_complex's $relation to $value.
		//Examples: 
		//Ownership:group 3; Match:weight > 20 -- Means all objects of weight greater than 20 belonging to group 3
		//Ownership:* Match:name = Sweater -- Means all objects named "Sweater" belonging to any group or user
		//Ownership:user 1; Match:* -- Means all objects belonging to user with id 1
		//Ownership:*; Match:* -- Means all objects in the database
		//Ownership:*; Match:weight > 20 weight < 30 -- Means all objects with weights between 20 and 30 (not inclusive)
		//NOTE: Statements are whitespace-sensitive. You should use collections/create to build collections, as their criterion statements
		//will be created automatically.
		//TODO: This could be cleaned up a bit, as it has evolved a number of times since it was created. Perhaps
		//a smaller, cleaner parser that uses regular expressions could be implemented, and/or the collection criterion
		//"language" specified above could be redesigned.
		if(!strstr($statement, ";") || !strstr(strtolower($statement), "ownership") || !strstr(strtolower($statement), "match")) {
			$this->output->set_output("There was a syntax error in your expression.");
			return false;
		}
		else {
			$temp = explode(";", $statement);
			
			//Split the 'ownership' part of the expression into $type and $id
			$lhs = explode(":", $temp[0]);
			if(trim(strtolower($lhs[0])) == "ownership") {
				$lhs = trim($lhs[1]);
			}
			else {
				$this->output->set_output("You must specify an ownership clause as the first part of your statement.");
				$lhs = "";
			}
			$lhs = explode(" ", $lhs);
			
			//Split the 'match' part of the expression into $field, $relation, and $value
			$rhs = explode(":", $temp[1]);
			if(trim(strtolower($rhs[0])) == "match") {
				$rhs = trim($rhs[1]);
			}
			else {
				$this->output->set_output("You must specify a match clause as the second part of your statement.");
				$rhs = "";
			}
			$rhs = explode(" ", $rhs);
			
			
			if(count($lhs) > 1) {
			$result['ownership_type'] = $lhs[0];
			$result['ownership_id'] = $lhs[1];
			}
			else {
				$result['ownership_type'] = "*";
			}
			
			//The parser takes match statements in threes, so
			//dummy values need to be filled in here
			if($rhs[0] == "*") {
				array_splice($rhs, 0, 0, "*");
				array_splice($rhs, 1, 0, "*");
			}
			if(count($rhs) % 3 == 0) {
				$i = 0;
				while(!empty($rhs)) {
				$result['match_values'][$i] = array_pop($rhs);
				$result['match_relations'][$i] = array_pop($rhs);
				$result['match_fields'][$i] = array_pop($rhs);
				$i++;
				}
			}
			else {
				$result['match_fields'][0] = "*";
			}
		
			return $result;
		}
	}
	
	/**
	 * Returns a subset of objects that match a collection criterion, but are not in a certain numerically-indexed
	 * range. Useful for pagination.
	 * @param string $statement The collection criterion
	 * @param int $limit The number of objects in the range
	 * @param int $offset The starting index of the range
	 * @return array The objects not in the given range, matching $statement
	 */
	function getObjectsNotInRange($statement, $limit, $offset) {
		$badRange = range($offset, $offset + $limit - 1);
		$allObjects = $this->getObjectsByStatement($statement);
		$i = 0;
		$subset = array();
		foreach($allObjects as $object) {
			if(!in_array($i, $badRange)) {
				$subset[] = $object;
			}
			$i++;
		}
		return $subset;
	}
	
	/**
	 * Takes a collection criterion, limit, offset, and sorting method, and returns the objects
	 * matching those parameters
	 * @param string $statement The collection criterion
	 * @param int $limit The number of objects in the range
	 * @param int $offset The starting index of the range
	 * @param string $sort The sorting method
	 * @return array The objects matching the criterion
	 */
	function getObjectsByStatement($statement, $limit=null, $offset=null, $sort = 'name ASC') {
		$query = $this->parseStatement($statement);
		if($query == false) {
			return array();
		}
		else {
			$this->db->select('objects_complex.*');
			$this->db->from('objects_complex');
			switch($query['ownership_type']) {
				case "*": //Any ownership type will do
					break;
				default: //Ownership type specified
					$this->db->join('permissions', 'objects_complex.oid = permissions.oid_number', 'inner');
					$this->db->where('permissions.id_number', $query['ownership_id']);
					$this->db->where('permissions.permission_type', $query['ownership_type'].'edit');
					break; 
			}
			for($i = 0; $i < count($query['match_fields']); $i++) {
				if($query['match_fields'][$i] != "*") {
					if(strtoupper(trim($query['match_relations'][$i] == "NOT_ILIKE")))
						$query['match_relations'][$i] = "NOT ILIKE";
					if(strtoupper(trim($query['match_relations'][$i])) == "ILIKE" ||
						strtoupper(trim($query['match_relations'][$i])) == "NOT ILIKE") {
						if(is_numeric($query['match_values'][$i])) {
							$query['match_relations'][$i] = "="; //Protect against numeric LIKE queries
						} else {
							$query['match_values'][$i] = '%'.$query['match_values'][$i].'%';
						}
					}
					if($query['match_fields'][$i] == "textual") {
						$this->db->where("name ".$query['match_relations'][$i], $query['match_values'][$i]);
						$this->db->or_where("description ".$query['match_relations'][$i], $query['match_values'][$i]);
					} else {
						$this->db->where($query['match_fields'][$i]." ".$query['match_relations'][$i], $query['match_values'][$i]);
					}
				}
			}
			//Make sure we don't bypass visibilty restrictions
			$this->db->where("visibility !=","private");
			$this->db->limit($limit, $offset);
			$this->db->order_by($sort);
			$objects = $this->db->get();
			$objects = $this->om->decorateObjects($objects->result());
			return $objects;
		}
	}
}
?>