<?php

/**
 * Representation of objects as model, handles transactions for
 * objects so they can be listed or edited. 
 * 
 * @author Sourcemap Team
 * @package sourcemap
 * @subpackage models
 */

class ObjectsModel extends Model {
	var $defaultfilter = array('objects_complex.flag !=' => 'objects_complex.temporary', 
							   'objects_complex.name !=' => '', 'objects_complex.visibility !=' => 'private');

	function ObjectsModel(){
		parent::Model();	
		$this->load->model(Array('UsersModel', 'AuthModel', 'TransportModel', 'GroupsModel', 'ProcessesModel', 'endoflifeModel'));
		$this->load->library('Log');
	}		

	/**
	 * Gets the information from the objects_complex table
	 * @param string $slug (alternatively, int id)
	 * @return array of objects
	 */
	public function getObject($slug) {
		$this->db->select('*');		
		$this->db->from('objects_complex');
		if(!(is_numeric($slug))) {$this->db->where('slug', $slug);}
		else { $this->db->where('oid', $slug);}
		return $this->_do_get_single(); 
	}

	/**
	 * Returns the total size of the objects table.
	 * @return int Number of objects.
	 */
	public function getObjectCount(){
		$this->db->where($this->defaultfilter);		
		$this->db->join("(select oid as pid, count(oid) as partcount from parts_instantiated group by pid) as parts_instantiated", "parts_instantiated.pid = objects_complex.oid", "left outer");
		$this->db->where("partcount > 1");		
		return $this->db->count_all_results('objects_complex');
	}
	
	/**
	 * Returns a random object.
	 * @return object a sourcemap object
	 */
	public function getRandomObject() {
		$this->db->select('*');		
		$this->db->from('objects_complex');
		$this->db->where($this->defaultfilter);
		
		$this->db->limit(1, 0);
		$this->db->order_by("", "RANDOM");
		
		$query = $this->db->get();
		return array_pop($query->result());
	}

	/**
	 * Gets the list of objects.
	 * @return Array Array of all objects, containing a parts array of instantiated parts.
	 * @param integer $limit Limit number of object.
	 * @param integer $offset Offset to return objects.
	 * @param string $sort How objects should be sorted (default category ASC, name ASC).
	 * @todo Add parameter for filtering.
	 */
	public function getObjects($limit = null, $offset = null, $sort = 'timemodified DESC, category ASC, name ASC'){
		$this->db->cache_on();
		
		$this->db->select("*", false);
		$this->db->from("objects_complex");
		$this->db->where($this->defaultfilter);

		// This join grabs the part count, it also essentially limits the results to objects with parts.
		$this->db->join("(select oid as pid, count(oid) as partcount from parts_instantiated group by pid) as parts_instantiated", "parts_instantiated.pid = objects_complex.oid", "left outer");
		$this->db->where("partcount > 1");	
		$this->db->limit($limit, $offset);
		$this->db->order_by($sort);
		
		$templist = $this->db->get();
		$templist = $templist->result();

		return $this->decorateObjects($templist);
	}
	
	/**
	 * Gets objects in a given oid list
	 * @param array $oidList
	 *@return array list of objects
	 */
	public function getObjectsByOids($oidList){
		$this->log->write_log('info', "getting objects by oid");
		$this->db->cache_on();
		
		$this->db->select('*');		
		$this->db->from('objects_complex');
		$this->db->where_in('oid', $oidList);
		return $this->_do_get_objects();
	}	
	
	/**
	 * Takes an array of vanilla objects and returns an array of objects with
	 * parts, comment count, favorite count, etc. added
	 * @param mixed $objectArray The source array of vanilla objects
	 */
	public function decorateObjects($objectArray) {

	  $oidarray = null;
	  foreach($objectArray as $k => $v) {
	    $oidarray[] = $v->oid;
	  }

		
	  $this->db->select('count(*), refid');
	  $this->db->where('type', "object");
	  if (isset($oidarray)) {
	  $this->db->where_in('refid', $oidarray);
	  }
	  $this->db->from('comments');
	  $this->db->group_by('refid');
	  $query = $this->db->get();
	  $comment_count_results = $query->result_array();
	  
	  /*  Set commment count to for all objects =0 
	   *  For every objectArray, match the refid and rewrite the comment count
	   */
	  
	  for($i=0; $i < count($objectArray); $i++) {	    

	    $objectArray[$i]->commentcount = 0;

	    foreach($comment_count_results as $commentcount_k => $commentcount_v) {
	      if($commentcount_v["refid"] == $objectArray[$i]->oid) {
		$objectArray[$i]->commentcount = $commentcount_v["count"];
		break;
	      }
	    } 
	  }

	  $this->db->select('count(*), oid');
	  if (isset($oidarray)) {
	    $this->db->where_in('oid', $oidarray);
	  }
	  $this->db->from('favorites');
	  $this->db->group_by('oid');
	  $query = $this->db->get();
	  $favorite_count_results = $query->result_array();
	  
	  /* Set favorite count to for all objects =0 
	   * Foe every objectArray, match the oid from favorites table and 
	   * rewrite the new favorite count
	   */

	  for($i=0; $i < count($objectArray); $i++) {	    

	    $objectArray[$i]->favcount = 0;

	    foreach($favorite_count_results as $favcount_k => $favcount_v) {
	      if($favcount_v["oid"] == $objectArray[$i]->oid) {
		$objectArray[$i]->favcount = $favcount_v["count"];
		break;
	      }
	    } 
	  }
	  
	  $objlist =$this->getCreatorAndParts($objectArray);
	  return $objlist;
	}


	/*
	 * Returns the list of the objects with updated objectArray
	 * @param array objectArray
	 */
	private function getCreatorAndParts ($objectArray) {
	  $objectlist = array();
	  $oidlist = array();
	  
	  /* Set creator for all objects 
	   * Set the values from object to Objectlist
	   * Get all the user ids from object
	   */
	  $creatorIds = null;
	  foreach ($objectArray as $object) {
	    $objectlist[$object->oid] = $object;	
	    $creatorIds[] = $object->creator; 
	    $oidlist[] = $object->oid;
	  }	
	 
	  
	  $users = $this->UsersModel->getUsersByIds($creatorIds); 

	  
	  /*For every objectArray element, check if the user id is same as the
	   * creator in the objects_complex table, if so then update the user info of the objectArray*/
	  for($i=0; $i < count($objectArray); $i++) {	    
	    
	    
	    foreach($users as $user) {
	      
	      if($user->id == $objectArray[$i]->creator) {
		$objectArray[$i]->user = $user;
		break;
	      }
	    } 
	  }	  
	  
	  /* Updating the objectArray with the parts_instantiated table */
	  if (sizeof($objectArray) > 0){		
	    $this->db->select('*');		
	    $this->db->from('parts_instantiated');			
	    if (isset($oidlist)) {
	    $this->db->where_in('oid', $oidlist);	
	    }
	    
	    $parts = $this->db->get();
	    $parts = $parts->result();
	    
	    foreach($objectlist as $object) {		
	      $object->parts = array();
	    }

	    foreach($parts as $part) {
	      if(isset($objectlist[$part->oid])) {
		if(!isset($objectlist[$part->oid]->parts)) {
		  $objectlist[$part->oid]->parts = array();
		} 
		array_push($objectlist[$part->oid]->parts, $part);
		
	      } else {
		$this->log->write_log('info', "didn't find any object for this oid, which should be impossible: ".$part->oid);
	      }
	      $part->transportemissions = $this->TransportModel->getEmissionsFromTransport($part->via); 
              
	    }

	    $this->db->select('*');
	    $this->db->from('process_instantiated');
	    if(isset($oidlist)) {
	      $this->db->where_in('oid', $oidlist);
	    }

	    $processes = $this->db->get();
	    $processes = $processes->result();

	    foreach($objectlist as $object) {
	      foreach($object->parts as $part) {
		$part->process = array();

		foreach($processes as $process) {
		  if($part->imid == $process->parts_inst_id) {
		    array_push($part->process, $process);
		  }
		}
	      }
	    }

	    $this->db->select('*');
	    $this->db->from('endoflife_instantiated');
	    if(isset($oidlist)) {
	      $this->db->where_in('oid', $oidlist);
	    }
	    
	    $endoflife = $this->db->get();
	    $endoflife = $endoflife->result();
	    
	    foreach($objectlist as $object) {
	      foreach($object->parts as $part) {
		$part->endoflife = array();
		
		foreach($endoflife as $end) {
		  if($part->imid == $end->parts_inst_id) {
		    array_push($part->endoflife, $end);
		  }
		}
	      }
	    } 
	    
	  }
	  return $objectlist;		
	}
	

	/**
	 * Gets the process data from parts_list slug array
	 */
	public function getProcessData($slug) {

	  $this->db->select('id');
	  $this->db->from('parts_list');
	  $this->db->where('slug', $slug);
	  $id = $this->db->get();
	  $part_id = array_pop($id->result())->id;
	  
	  $this->db->select('*');
	  $this->db->from('processes');
	  $this->db->join("applicable_parts", "applicable_parts.proc_id = processes.id", "inner");
	  $this->db->where_in('parts_id', $part_id);
	  $query = $this->db->get();
	  $result = $query->result();

	  return $result;
	}

	
	public function getEndoflifeData($slug) {
	  
	  $this->db->select('id');
	  $this->db->from('parts_list');
	  $this->db->where('slug', $slug);
	  $id = $this->db->get();
	  $part_id = array_pop($id->result())->id;
	  
	  $this->db->select('*');
	  $this->db->from('endoflife');
	  $this->db->join("endoflife_parts", "endoflife_parts.endoflife_id = endoflife.id", "inner");
	  $this->db->where_in('parts_id', $part_id);
	  $query = $this->db->get();
	  $result = $query->result();
	  return $result;
	  }
	


	/**
	 * Updates an objects permissions.
	 * @param $type string the permission type (useredit, groupedit, everyoneedit)
	 * @param $oid integer the oid to update
	 * @param $id the user or group id (everyone uses 0)
	 */
	public function updatePermissions($type, $oid, $id="0") {
		$this->db->where('oid_number', $oid);
		$this->db->delete('permissions');

		$info = array('permission_type' => addslashes($type),
			'id_number' => addslashes($id),
			'oid_number'=>addslashes($oid)

		);
		$this->db->insert('permissions', $info);
	}

	/**
	 * Updates an objects visibility
	 * @param $visibility string either public or private
	 * @param $oid integer the oid to update
	 */
	public function updateObjectVisibility($visibility, $oid) {	
		$data = array('visibility'=>$visibility);
		$this->db->where('oid', $oid);
		$this->db->update('objects_complex', $data);	  
	}
	
	/**
	 * Gets the count of all the objects matched with uid from the favorites table with the $id.
	 * @param int $id
	 * @limit int 10
	 * @return array of objects
	 */
	
	function getFavoritedObjectsByUserID($id, $limit=10){
		$table = "favorites";
		$this->db->select("objects.*, favorites.*");
		$this->db->from("objects_complex objects");
		$this->db->join("favorites", "favorites.oid = objects.oid", "inner");
		
		$this->db->where('favorites.uid', $id);
		
		$this->db->limit($limit);
		
		$objects = $this->db->get();
		$objects = $objects->result();
		return $this->decorateObjects($objects);
	
	}

	
	/**
	 * Returns the array of objects by username 
	 * @param string $username
	 * @limit int $limit=10
	 */
	function getCommentedObjectsByUserName($username, $limit=10){
		$table = "comments";
		$this->db->select("objects.*, comments.*");
		$this->db->from("objects_complex objects");
		$this->db->join("comments", "comments.refid = objects.oid", "inner");
		
		$this->db->where('comments.author', $username);
		
		$this->db->limit($limit);
		
		$objects = $this->db->get();
		$objects = $objects->result();

		return $this->decorateObjects($objects);
		
	}
	/**
	 * Gets the list of all objects by user id.
	 * @return Array Array of all objects, containing a parts array of instantiated parts.
	 * @param integer $limit Limit number of object.
	 * @param integer $offset Offset to return objects.
	 * @param string $sort How objects should be sorted (default category ASC, name ASC).
	 */
	function getObjectsByUserID($id, $limit = 30, $sort = 'timemodified DESC, category ASC, name ASC'){

	  // Get group ids where user is a member
 	  $this->db->select('group_id');
 	  $this->db->from('members');
 	  $this->db->where('user_id', $id);
 	  $query = $this->db->get();
 	  $group_id = $query->result_array();

	  // Build an array of group_ids
	  $group_ids = null;
	  foreach($group_id as $k => $v) {
	    $group_ids[] = $v["group_id"];
	  }

	  // Get object ids for the group ids
	  if(count($group_ids) > 0) {
		  $this->db->select('oid_number');
		  $this->db->from('permissions');	  	
		  $this->db->where('permission_type', 'groupedit');	  	
		
		  $this->db->where_in('id_number', $group_ids);
		  $query = $this->db->get();
		  $object_id = $query->result_array();

		  // Build an array of object_ids
	 	  $object_ids = null;
		  foreach($object_id as $k => $v) {
		    $object_ids[] = $v["oid_number"];
		  }
	  }


	  $this->log->write_log('info', "getting objects for user id ".$id);
	  $this->db->select('*');		
	  $this->db->from('objects_complex');

	  $this->db->where('flag !=', 'temporary');		
	  $this->db->where('name !=', '');
	  $this->db->where('creator', $id); 
	  if(isset($object_ids)) {
	    $this->db->or_where_in('oid', $object_ids);
	  }
	  $this->db->limit($limit);
	  $this->db->order_by($sort);
	  
	  $temp = $this->_do_get_objects();			

	  return $temp;
	}


	function _clearCachedObjects() {
		$this->db->cache_delete_all(); // TODO This is temporary.
		$this->db->cache_delete("objects","index");
		$this->db->cache_delete("default","index");
		$this->db->cache_delete("feed","index");		
	}
		
	/**
	 * This creates the url_title or slug
	 */
	function generateSlug($object, $usePrevSlug = false) {
		//Handle slugs
		if(!isset($object->slug) || $object->slug == '"' || $object->slug == "") {
			$slug = @url_title(utf8_decode(strtolower($object->name)));
			$slug = str_replace(".", "-", $slug); 
			$test = $this->getObject($slug);
			$slugCount = 0; 
			while(isset($test)) { $slugCount++; $test = $this->getObject($slug."-".$slugCount); }
			if ($usePrevSlug){ $slugCount--; }
			if ($slugCount > 0) { $slug = $slug."-".$slugCount; }
		} 
		else { $slug = $object->slug;}
		return $slug;
	}
	

	/**
	 * Saves all the changes made into a variable 
	 * @param $object object the sourcemap object.
	 * @return the slug updated or inserted for the object.
	 */
	public function saveObject($object, $as_temporary=false) {
		if (isset($object->oid)){ $this->log->write_log('info', "saving object ".$object->oid);}
		
		$slug = $this->generateSlug($object, false);

		$this->addProcesses($object);
		$this->addEndoflife($object);
		
		$this->_clearCachedObjects();
		$currentTime = gmdate("Y-m-d H:i:s", time());
		$newflag = $as_temporary ? 'temporary' : '';
		
		if($object->flag == "added") { return $this->_createObject($object, $slug, $newflag, $currentTime);}
		else { return $this->_updateObject($object, $slug, $newflag, $currentTime);	} 
	}
	
	/**
	 * Returns true if the object is ready to be saved, and false if something is missing
	 */
	function validate($object){
	  return true;
	}
	
	/**
	 * Returns empty string if the object is ready to be saved, and description of what's missing if something is missing
	 */
	function explain_validation_errors(){
	  return "";
	}
	
	/**
	 * Returns the linked object for object expansion.
	 * @param $linkid string the linked object id.
	 * @return object the complete sourcemap object.
	 */
	function getObjectByLinkID($linkid) {
	  $slug = explode("/", $linkid);
	  $slug = $slug[1];
	  
	  $this->db->select('*');		
	  $this->db->from('objects_complex');
	  $this->db->where('slug', $slug);
	  $query = $this->db->get();
	  
	  $objects = $this->decorateObjects($query->result());
	  $object = array_pop($objects);
	  return $object;
	}
	
	/**
	 * Returns the object json encoded for javascript editing.
	 * @param $object object Sourcemap object to convert to JSON
	 * @return string json object as a string	 
	 */
	function generateJSONObject($object) {
	  $json = "var objectConfig = new Sourcemap.Object(".json_encode($object).");";
	  $json .= "objectConfig.parts = [";
	  foreach($object->parts as $part) { 
	    $json .= json_encode($part) . ",";
	  }
	  $json .= "];";
	  $json .= "objectConfig.partcount = ".(count($object->parts)).";";

	  /*objectConfig.part[i].process[i]*/
	  foreach($object->parts as $part) {
	  $json .= "objectConfig.parts.process = [";
	  foreach($part->process as $process) { $json .= json_encode($process) . ",";}
	  $json .= "];";
	  $json .= "objectConfig.processcount = ".(count($part->process)).";"; 
	  }
	  
	  /*objectConfig.part[i].endoflife[i]*/ 
	  foreach($object->parts as $part) {
	    $json .= "objectConfig.parts.endoflife = [";
	    foreach($part->endoflife as $endoflife) { $json .= json_encode($endoflife) . ",";}
	    $json .= "];";
	    $json .= "objectConfig.endoflifecount = ".(count($part->endoflife)).";"; 
	  }
	  return $json;
	}


	/**
	 * Generates an array of JSON Objects from an array of Sourcemap Objects
	 * @param $varName string The name of the JSON Object to be generated
	 * @param $objects object Sourcemap objects to convert to JSON
	 * @param $update boolean Determines whether to create a new variable or update an existing one.
	 * @return string json array as a string	
	 */
	function generateJSONObjectArray($varName, $objects, $update = false) {
		if(!$update) { $json = "var "; }
		
		$json = "$varName = {}; ";
		foreach($objects as $object) {
			$json .= "$varName"."[$object->oid] = new Sourcemap.Object(".json_encode($object).");";
			$json .= "$varName"."[$object->oid].parts = [";
			if(!empty($object->parts)) {
				$isFirst = true;
				foreach($object->parts as $part) {
					if(!$isFirst) { $json .= ",";}
					$json .= json_encode($part);
					$isFirst = false;
				}
			}
			$json .= "];";
			$json .= "$varName"."[$object->oid].partcount = ".(count($object->parts))."; ";
		}
		return $json;
	}
	
	/**
	 * Creates a blank json for an object to start sourcemap creation.
	 * @param $type string the template type
	 * @param $creator integer the creator user
	 * @return string json blank as a string
	 */
	public function generateJSONBlank($type, $creator) {
		$json = "var objectConfig = new Sourcemap.Object();";
		$json .= "objectConfig.parts = [];";
		$json .= "objectConfig.partcount = 0;";
		$json .= "objectConfig.flag = 'added';";		
		$json .= "objectConfig.type = '".$type."';";
		$json .= "objectConfig.latlon = '0|0';";		
		$json .= "objectConfig.creator = ".$creator.";";

		$json .= "objectConfig.parts.process = [];"; 
		$json .= "objectConfig.processcount =0;";  

		$json .= "objectConfig.parts.endoflife = [];";
		$json .= "objectConfig.endoflifecount =0;";
		return $json;
	}

	/**
	 * Gets a single object
	 * @return object a single object
	 */
	private function _do_get_single(){ 
		$arr =  $this->_do_get_objects(); 
		$rv = null; 
		foreach($arr as $object){ $rv = $object;} 
		return $rv;             
	}
	
	/**
	 * Returns the list of objects 
	 * @return array the decorated list of objects
	 */
	private function _do_get_objects(){ 
	  //$this->log->write_log('info', "finding objects");
	  $templist = $this->db->get();
	  $templist = $templist->result();
	  return $this->decorateObjects($templist);
	}
	
	/**
	 * Creates a new object in the database.
	 * @param $object object the sourcemap object.
	 * @param $slug string the url safe slug name
	 * @param $newflag flag representing object add status
	 * @param $currentTime string the system time.
	 * @return the slug updated for the object.
	 */
	private function _createObject($object, $slug, $newflag, $currentTime) {
		$this->db->trans_start();
		
		$data = array(
			'name' => @utf8_decode($object->name),				'description' => (@utf8_decode($object->description)),
			'type' => @utf8_decode($object->type),				'upc' => $object->upc,
			'imagelink' => @utf8_decode($object->imagelink),	'flag' => $newflag,			
			'creator' => $object->creator,						'origin' => @utf8_decode($object->origin),
			'latlon' => $object->latlon,						'timemodified' => $currentTime,
			'slug' => $slug
		);
		
		$this->db->insert('objects_complex', $data);
		$query = $this->db->query('SELECT LAST_VALUE FROM public.objects_complex_oid_seq');
		$row = $query->row_array();		
		$object->oid = $row['last_value'];
		$this->_initializePermissions($object);

		$this->db->trans_complete(); 
		$this->db->trans_off();     
							
		// TODO: need to add parts, too.  Refactor it out of the else statement so it happens every time no matter what
		$this->_save_parts($object);
		
		$this->db->select("slug");
		$this->db->from("objects_complex");
		$this->db->where('oid', $row['last_value']);

		$object = $this->db->get();
		$object = array_pop($object->result());
		return $object->slug;
	}
	
	/**
	 * Updates an object that already exists.
	 * @param $object object the sourcemap object.
	 * @param $slug string the url safe slug name
	 * @param $newflag flag representing object add status
	 * @param $currentTime string the system time.
	 * @return the slug updated for the object.
	 */
	private function _updateObject($object, $slug, $newflag, $currentTime) {
		$data = array(
			'name' => @utf8_decode($object->name),				'description' => (@utf8_decode($object->description)),
			'type' => @utf8_decode($object->type),				'upc' => $object->upc,
			'imagelink' => @utf8_decode($object->imagelink),	        'flag' => $newflag,			
			'creator' => $object->creator,					'category' => @utf8_decode($object->category),
			'origin' => @utf8_decode($object->origin),			'latlon' => $object->latlon,			
			'destination' => @utf8_decode($object->destination),            'weight' => $object->weight,
			'emissions' => $object->emissions,				'embodied' => $object->embodied,
			'transport' => $object->transport,				'shipping' => $object->shipping,
			'via' => @utf8_decode($object->via),				'lifetime' => $object->lifetime,
			'usagetype' => @utf8_decode($object->usagetype),	        'usageenergy' => $object->usageenergy,
			'usageemissions' => $object->usageemissions,		        'timemodified' => $currentTime,
			'visibility' => $object->visibility,				'showfootprint' => $object->showfootprint,
			'showembodied' => $object->showembodied,			'showtransport' => $object->showtransport,
			'showprocess' => $object->showprocess,				'showendoflife' => $object->showendoflife,
			'showlastleg' => $object->showlastleg,				'showuse' => $object->showuse,
			'showstats' => $object->showstats,				'slug' => $slug,
			'process' => @$object->processEmissions,           'endoflife' => @$object->endoflifeEmissions
		);

		$this->db->where('oid',$object->oid);
		$this->db->update('objects_complex', $data);
		$this->_save_parts($object);
	}



	function getEndoflifeByOid($oid) {
	  $this->db->select('*');
	  $this->db->from('endoflife_instantiated');
	  $this->db->where('oid', $oid);
	  
	  $query = $this->db->get();
	  $result = $query->result();
	  
	  return $result;
	  
	}

	
	/*
	 * Updates the endoflife_instantiated with the endoflife value
	 */
	private function addEndoflife($object) {
	  $oid = @utf8_decode($object->oid);
	  $parts = $object->parts;

	  foreach($parts as $part) { 
	    if(isset($part->endoflife)) {
	      $name = $part->name;
	      $endoflife = $part->endoflife;

	      foreach($endoflife as $p => $q) {

		$flag = @utf8_decode($q->flag);
		if(isset($q->emissions)) {
		  $emissions = @utf8_decode($q->emissions);
		} else {
		  $emissions = 0;
		}
		$unit = @utf8_decode($q->unit);
		$endoflife_type = @utf8_decode($q->type);
		
		if($flag == false) {
		  $check = $this-> CheckEndoflife($endoflife_type);
		  
		  if($check == true) {
		    
		    $pid = @utf8_decode($part->imid);
		      
		      $data= array('oid' => addslashes($oid),
				   'name'=> addslashes($name),
				   'type' => addslashes($endoflife_type),
				   'emissions' => addslashes($emissions),
				   'parts_inst_id' => addslashes($pid),
				   'unit'=> addslashes($unit)
				   );
		      $this->db->insert('endoflife_instantiated', $data);
		  } else {
		    
		    $pid = @utf8_decode($part->imid);
		    
		    $newdata=  array('oid' => addslashes($oid),
				     'name'=> addslashes($endoflife_type),
				     'type' => addslashes($endoflife_type),
				     'emissions' => addslashes($emissions),
				     'parts_inst_id' => addslashes($pid),
				     'unit'=> addslashes($unit)
				     );
			$this->db->where('type', $endoflife_type);
			$this->db->update('endoflife_instantiated', $newdata);
		  }  
		
		} else {
		  $this->db->where('type', $endoflife_type);
		  $this->db->delete('endoflife_instantiated');
		        	       
		}
	      }
	    }
	  }
	}

       private function checkEndoflife($type) {
	 $this->db->select('id');
	 $this->db->from('endoflife_instantiated');
	 $this->db->where('type', $type);
	 $query = $this->db->get();
	 $result = array_pop($query->result());
	 if ($result == null) {$check = true;}
	 else {$check = false;}
	 return $check;
       }

	private function getEndoflifeByType($endoflife_type) {
	  $this->db->select('*');
	  $this->db->from('endoflife');
	  $this->db->where('type', $endoflife_type);
	     $query = $this->db->get();
	     $result = $query->result_array();
	     if($result == null) {
	       $result = false;
	     }
	     return $result;
	     
	}
	
	
	


	/*
	 * Updates the process_instantiated and processes table with the new process or factor, emissions
	 * @param $object object 
	 */
	private function addProcesses($object) {
	  $oid = @utf8_decode($object->oid);
	  $parts = $object->parts;
	  foreach ($parts as $part) {
	      if (isset($part->process)) {
		$process = $part->process;

		foreach($process as $p => $q) {
		  
		  $flag = @utf8_decode($q->flag);
		  $emissions = @utf8_decode($q->emissions);
		  $unit = @utf8_decode($q->unit);
		  $factor = @utf8_decode($q->factor);
		  $label = @utf8_decode($q->label);
		 
      
		  if ($flag == false){
		      $process_name = @utf8_decode($q->name);
		      
		      $check = $this->CheckProcess($process_name);
		      
		      if($check == true) {
			$result = $this->getProcessesByName($process_name);
			
			if ($result != false) {
			foreach($result as $k => $v) {

			  $pid = @utf8_decode($part->imid); 
			  
			  $data = array('oid' =>addslashes($oid),
					'name' => addslashes($v["name"]),
					'description' => addslashes($v["description"]),
					'type' => addslashes($v["type"]),
					'emissions' =>addslashes($emissions),
					'parts_inst_id' =>addslashes($pid),
					'unit' => addslashes($unit),
					'factor' => addslashes($factor)
					
					);
			  
			  $this->db->insert('process_instantiated', $data);
			}
			} else {

			  $pid = @utf8_decode($part->imid); 
			  $type = "new";
			  $newdata = array ('oid' => addslashes($oid),
					    'name' => addslashes($process_name),
					    'type' => addslashes($type),
					    'emissions' => addslashes($emissions),
					    'parts_inst_id' => addslashes($pid),
					    'unit' => addslashes($unit),
					    'factor' => addslashes($factor)
					    );
			  $this->db->insert('process_instantiated', $newdata);
			  
			}

		       } else { 
			 if($label == 1) {
			   $info = array (
					  'emissions' => addslashes($emissions),
					  'factor' => addslashes($factor),
					  );
			   $this->db->where('name', $process_name);
			   $this->db->update('process_instantiated', $info);
			   }
		       }

		   } else { 
		     $processName = $q->name;
		     $this->deleteProcess($processName);

		   }

		 }

	       }

	   }

	 }

	 private function deleteProcess($processName){
	   $this->db->where('name', $processName);
	   $this->db->delete('process_instantiated');
	 }

	  private function getProcessesByName($name) {
	     $this->db->select('*');
	     $this->db->from('processes');
	     //$this->db->where_in('name', $names);
	     $this->db->where('name', $name);
	     $query = $this->db->get();
	     $result = $query->result_array();
	     if($result == null) {
	       $result = false;
	     }
	     return $result;

	   }

	   private function getPartId($name) {
	     $this->db->select('imid');
	     $this->db->from('parts_instantiated');
	     $this->db->where('name', $name);
	     $query = $this->db->get();
	     $result = array_pop($query->result());
	     return $result->imid;
	   }

	   private function checkProcess($name) {
	     $this->db->select('id');
	     $this->db->from('process_instantiated');
	     $this->db->where('name', $name);
	     $query = $this->db->get();
	     $result = array_pop($query->result());
	     if ($result == null) {$check = true;}
	     else {$check = false;}
	     return $check;
	   }


	   function getProcessesByOid($oid) {
	     $this->db->select('*');
	     $this->db->from('process_instantiated');
	     $this->db->where('oid', $oid);

	     $query = $this->db->get();
	     $result = $query->result();

	     return $result;

	   }

	public function getPartLocationData($get_part_location, $material_name) {
	  $get_part_location = strtolower($get_part_location);
	  $get_part_location = ucfirst($get_part_location);
	  $material_name = strtolower($material_name);
	  $this->db->select('displaylocation');
	  $this->db->from('source_locations');
	  $this->db->like('displaylocation', $get_part_location, 'after'); 
	  $this->db->where('partname', $material_name);
	  $query = $this->db->get();
	  $result = $query->result();
	  return $result;
	}


	 /**
	  * Saves the updated information in the parts_instatiated table
	  * @param $object object the sourcemap object.	
	  */
	 private function _save_parts($object){
		 $parts = $object->parts;
		 foreach($parts as $part) {
			 if(!(isset($part->flag))) {  $part->flag = "unchanged"; }

			 if($part->flag != "unchanged") {			
				 if ($part->flag == "remove") {
					 $this->db->where('imid', $part->imid);
					 $this->db->delete('parts_instantiated');
				 } else {
					if(!isset($part->processemissions)) {
						$part->processemissions = 0;
					}
					if(!isset($part->endoflifeemissions)) {
		$part->endoflifeemissions = 0;
					}
					 $data = array(
						 'oid' => $object->oid,							'linkid' => $part->linkid,																	
						 'name' => @utf8_decode($part->name),			'description' => @utf8_decode($part->description),
						 'origin' => @utf8_decode($part->origin),		'latlon' => $part->latlon,						
						 'weight' => $part->weight,						'rawemissions' => $part->rawemissions,
						 'emissions' => $part->emissions,				'shipping' => $part->shipping,
						 'processemissions' => $part->processemissions, 'endoflifeemissions' => $part->endoflifeemissions,
						 'via' => @utf8_decode($part->via),					
					 );

					 if($part->flag == "changed") {
						 $this->db->where('imid',$part->imid);
						$this->db->update('parts_instantiated', $data);
					} else if ($part->flag == "add") { 
						$this->db->insert('parts_instantiated', $data);
					}
				}
			} // If not unchanged
		} // Foreach part		
	}
	
	/**
	 * Sees if permissions exist for the current object.
	 * @param $oid integer oid of the object to check.	
	 * @return boolean true if permissions are assigned, false otherwise.
	 */
	private function _hasPermissions($oid) {
		$this->db->select('*');
		$this->db->from('permissions');
		$this->db->where('oid_number', $oid);
		$query = $this->db->get();
		if($query->num_rows() > 0) { return true;} 
		else { return false; }
	}
	
	/**
	 * Creates a base permission for a new object (useredit for the creator)
	 * @param $object object the sourcemap object.	
	 */
	private function _initializePermissions($object) {
		if(!$this->_hasPermissions($object->oid)) {
			$data = array ('permission_type' => "useredit",
				 'id_number'=> $object->creator,
				 'oid_number' => $object->oid);
			$this->db->insert('permissions', $data);
		}
	}
}
?>